/** \file GSNode.h */
//
//  GSNode.h
//  GlyphsCore
//
//  Created by Georg Seifert on 05.04.09.
//  Copyright 2009 schriftgestaltung.de. All rights reserved.
//

#import <GlyphsCore/GSElement.h>
#import <Cocoa/Cocoa.h>
#import <GlyphsCore/GSUserDataProtocol.h>

NS_ASSUME_NONNULL_BEGIN

@class GSPath;

/**
 An enumeration of node and connection types.
 */
typedef NS_ENUM(uint8_t, GSNodeType) {
	/// Move
	MOVE = 17,
	/// Line
	LINE = 1,
	/// Curve
	CURVE = 35,
	/// Offcurve
	OFFCURVE = 65,
	/// Quadratic curve
	QCURVE = 36,
	HOBBYCURVE = 37,
	/// Sharp connection
	SHARP = 0,
	/// Smooth connection
	SMOOTH = 100,
	SPECIAL = 101,
	TANGENTSMOOTH = 102
};

/** The class defining the node object

 It is a subclass of GSElement
 */
@interface GSNode : GSElement <NSCoding, NSCopying, GSUserDataProtocol> {
	NSMutableDictionary *_userData;
  @public
	GSNodeType _type;
	GSNodeType _connection;
}

/// More specifically typed convenience getter for the generic `parent` property declared by `GSShape`.
@property (weak, nonatomic, nullable, readonly) GSPath *parentPath;

/// node type
@property (assign, nonatomic) GSNodeType type;
@property (readonly) char typeTag;

/// connection type
@property (assign, nonatomic) GSNodeType connection;

- (CFStringRef)elementString __attribute__((cf_returns_retained));

- (instancetype)initWithElementString:(NSString *)elementString;

- (BOOL)isEqualToNode:(GSNode *)other;

- (BOOL)canAttachCorner;
/**
 convenient init
 
 @param position the position
 @param type the type of the node (LINE, CURVE, QCURVE)
 @param connection the connection (SMOOTH, SHARP)
 */
- (instancetype)initWithPosition:(NSPoint)position type:(GSNodeType)type connection:(GSNodeType)connection;

/** toggles the connection type between SHARP and SMOOTH */
- (void)toggleConnection:(BOOL)makeSmooth;

/// automatically set the connection for handle depending on the angle
- (void)checkConnection;

/** Convenient method to make this the first node in the path.*/
- (void)makeNodeFirst;

- (NSUInteger)nodeIndex;

- (nullable GSNode *)nextOncurveNode;
@end
NS_ASSUME_NONNULL_END
